@@ -1,4 +1,4 @@
-from .base import *
+from .base import *  # noqa: F403
 from .eegnet import EEGNet
 from .shallownet import ShallowNet
-from .convca import ConvCA
\ No newline at end of file
+from .convca import ConvCA
